<?php

/********************************************************************
 * 
 * 画像リサイズ処理関数群
 * 
 ********************************************************************/

//指定された幅または高さに画像を縮小して出力する。
//入力と同じフォーマットで出力される。
//
//EXIF付きのJPEG画像の場合、EXIFにそって画像が回転・反転される。
//
//幅と高さ両方を指定した場合は縦横長い方の辺が指定された幅または高さにリサイズされる。
//
//$inputPath : 入力されるファイルパス
//$new_width : リサイズ後の幅。こちらが0の場合は高さに合わされる。
//$new_height: リサイズ後の高さ。こちらが0の場合は幅に合わせられる。
//$outputPath: 出力されるファイルパス
//$quality: jpg品質
function createResizeImage($inputPath, $new_width, $new_height, $outputPath, $quality)
{

    //オリジナルデータを読み取り
    $file_contents = file_get_contents($inputPath);
    if ($file_contents == false) {
        //エラー
        throw new Exception("ファイルの読み取りに失敗しました");
    }
    $original_image = imagecreatefromstring($file_contents);
    $original_width = imagesx($original_image);
    $original_height = imagesy($original_image);

    $resize_height = $original_height;
    $resize_width = $original_width;

    if ($new_width != 0 && $new_height != 0) {
        //両方に値が指定された場合は長辺を指定の長さにリサイズ
        if ($original_width > $original_height) {
            //横を基準にリサイズ
            $new_height = 0;
        } else {
            //タテを基準にリサイズ
            $new_width = 0;
        }
    }
    if ($new_width != 0) {
        //幅指定でリサイズ
        if ($new_width < $original_width) {
            $resize_width = $new_width;
            $resize_height = round($original_height * $resize_width /  $original_width);
        }
    } else if ($new_height != 0) {
        //高さ指定でリサイズ
        if ($new_height < $original_height) {
            $resize_height = $new_height;
            $resize_width = round($original_width * $resize_height /  $original_height);
        }
    }

    $is_jpeg = preg_match('/\.(jpe?g|jfif|pjpeg|pjp)/i', $outputPath);

    if ($resize_height == $original_height && $resize_width == $original_width && !$is_jpeg) {
        //リサイズしないサイズ & Jpegでない場合は品質劣化を避けるためコピー
        copy($inputPath, $outputPath);
    } else {

        // リサイズ
        $resize_image = imagecreatetruecolor($resize_width, $resize_height);
        //imagecolortransparent($resize_image, imagecolorallocate($resize_image, 0, 0, 0));
        imagefill($resize_image, 0, 0, imagecolorallocate($resize_image, 255, 255, 255)); // 画像を白色で塗りつぶす
        imagecopyresampled($resize_image, $original_image, 0, 0, 0, 0, $resize_width, $resize_height, $original_width, $original_height);
        if ($is_jpeg) {
            // EXIFデータを取得
            try {
                $exif_data = exif_read_data($inputPath, "EXIF");
            } catch (Exception $ex) {
                $exif_data = array();
            }
            //EXIFデータがある場合はそれに基づいて回転・反転する
            if (isset($exif_data['Orientation']) && $exif_data['Orientation'] <= 8) {
                $mode = null;

                // 画像の回転角度を決定
                $orientation = $exif_data['Orientation'];
                $degrees = [0, 0, 0, 180, 0, 270, 270, 90, 90][$orientation];
                $degrees2 = [0, 0, 0, 0, 0, 0, 0, 1, 1][$orientation];

                // 画像の反転モードを決定
                $orientation = $exif_data['Orientation'];
                $mode = [null, null, IMG_FLIP_HORIZONTAL, null, IMG_FLIP_VERTICAL, IMG_FLIP_HORIZONTAL, null, IMG_FLIP_VERTICAL, null][$orientation];

                // 画像の反転
                if ($mode !== null) {
                    $resize_image = imageflip($resize_image, $mode);
                }

                // 画像の回転
                if ($degrees > 0) {
                    $resize_image = imagerotate($resize_image, $degrees, 0);
                }

                // 返却値の回転
                if ($degrees2 == 1) {
                    $tmp = $resize_width;
                    $resize_height = $resize_width;
                    $resize_width = $tmp;
                }

                //保存
                if ($resize_height == $original_height && $resize_width == $original_width && $mode == null && $degrees == 0 && $degrees2 == 0) {
                    //リサイズも回転もない場合そのまま保存
                    copy($inputPath, $outputPath);
                } else {
                    imagejpeg($resize_image, $outputPath, $quality);
                }
            } else  if ($resize_height == $original_height && $resize_width == $original_width) {
                //リサイズしないサイズの場合そのまま保存
                copy($inputPath, $outputPath);
            } else {
                //リサイズしたものを保存
                imagejpeg($resize_image, $outputPath, $quality);
            }
        } elseif (preg_match('/\.png$/i', $outputPath)) {
            imagepng($resize_image, $outputPath);
        } elseif (preg_match('/\.gif$/i', $outputPath)) {
            imagegif($resize_image, $outputPath);
        } else {
            //不明の拡張子の場合はとりあえずJpegにしておく
            imagejpeg($resize_image, $outputPath, $quality);
        }

        imagedestroy($resize_image);
    }
    imagedestroy($original_image);

    return array("width" => $resize_width, "height" => $resize_height);
}
